#version 450

in vec2 uv;

uniform mat4 inverse_view;
uniform mat4 inverse_projection;

uniform sampler2DMS albedo;
uniform sampler2DMS depth;
uniform sampler2D lighting;

uniform float fog_start;
uniform float fog_end;
uniform vec3 fog_color;

uniform vec3 sky_color;

out vec3 color;

// Converts a color from linear light gamma to sRGB gamma
vec3 fromLinear(vec3 linearRGB)
{
    bvec3 cutoff = lessThan(linearRGB, vec3(0.0031308));
    vec3 higher = vec3(1.055)*pow(linearRGB, vec3(1.0/2.4)) - vec3(0.055);
    vec3 lower = linearRGB * vec3(12.92);

    return mix(higher, lower, cutoff);
}

vec3 calculate_view_position(float depth)
{
	vec4 viewPos = inverse_projection * vec4(uv.x * 2 - 1., uv.y * 2 - 1., 2.0 * depth - 1.0, 1.0);
	return viewPos.xyz / viewPos.w;
}

void main() {
	vec3 lighting = texture(lighting, uv).rgb;

	float nearest_depth = 1.1;
	float fog_depth = 0.;
	int fog_count = 0;
	vec3 collected_color = vec3(0., 0., 0.);
	for (int i = 0; i < gl_NumSamples; i++) {
		float depth = texelFetch(depth, ivec2(uv * textureSize(depth)), i).x;
		vec3 albedo = texelFetch(albedo, ivec2(uv * textureSize(albedo)), i).rgb;

		if (depth >= 1.) {
			discard;
		} else {
			if (depth < nearest_depth) {
				nearest_depth = depth;
			}
			
			collected_color += albedo * lighting;
			fog_depth += length(calculate_view_position(depth));
		}

		fog_count += 1;
	}

	collected_color /= gl_NumSamples;

	gl_FragDepth = nearest_depth;
	
	if (fog_count > 0) {
		fog_depth /= fog_count;
	}

	// Apply fog
	float fog_factor = smoothstep(fog_start, fog_end, fog_depth);
	collected_color = mix(fromLinear(collected_color), fog_color, fog_factor);

	color = collected_color;
}