#version 450
layout (points) in;
layout (triangle_strip, max_vertices = 4) out;

uniform mat4 projection;
uniform mat4 view;
uniform float scale;

in VS_OUT {
	uint voxelTexture;
	uint face;
	vec3 offset;
} gs_in[];

out vec2 uv_out;
out uint voxelTexture;
out vec3 normal_out;
out float lighting_out;

const vec3 faceNormals[10] = vec3[10](
	// East
	vec3(1, 0, 0),

	// Up
	vec3(0, 1, 0),
	
	// North
	vec3(0, 0, 1),

	// West
	vec3(-1, 0, 0),

	// Down
	vec3(0, -1, 0),

	// South
	vec3(0, 0, -1),

	// Diagonal 1 (0)
	vec3(0.70710677, 0, 0.70710677),

	// Diagonal 1 (1)
	vec3(0.70710677, 0, 0.70710677),

	// Diagonal 2 (0)
	vec3(0.70710677, 0, 0.70710677),

	// Diagonal 2 (1)
	vec3(0.70710677, 0, 0.70710677)
);

// Offsets for our vertices drawing this face
const vec3 facePositions[40] = vec3[40]
(
	// East
	vec3(1.0, 0.0, 0.0),
	vec3(1.0, 1.0, 0.0),
	vec3(1.0, 0.0, 1.0),
	vec3(1.0, 1.0, 1.0),

	// Up
	vec3(0.0, 1.0, 0.0),
	vec3(0.0, 1.0, 1.0),
	vec3(1.0, 1.0, 0.0),
	vec3(1.0, 1.0, 1.0),

	// North
	vec3(1.0, 0.0, 1.0),
	vec3(1.0, 1.0, 1.0),
	vec3(0.0, 0.0, 1.0),
	vec3(0.0, 1.0, 1.0),

	// West
	vec3(0.0, 0.0, 1.0),
	vec3(0.0, 1.0, 1.0),
	vec3(0.0, 0.0, 0.0),
	vec3(0.0, 1.0, 0.0),

	// Down
	vec3(0.0, 0.0, 1.0),
	vec3(0.0, 0.0, 0.0),
	vec3(1.0, 0.0, 1.0),
	vec3(1.0, 0.0, 0.0),

	// South
	vec3(0.0, 0.0, 0.0),
	vec3(0.0, 1.0, 0.0),
	vec3(1.0, 0.0, 0.0),
	vec3(1.0, 1.0, 0.0),

	// Diagonal 1 (0)
	vec3(0.075, 0.0, 0.075),
	vec3(0.075, 1.0, 0.075),
	vec3(0.925, 0.0, 0.925),
	vec3(0.925, 1.0, 0.925),

	// Diagonal 1 (1)
	vec3(0.925, 0.0, 0.925),
	vec3(0.925, 1.0, 0.925),
	vec3(0.075, 0.0, 0.075),
	vec3(0.075, 1.0, 0.075),

	// Diagonal 2 (0)
	vec3(0.925, 0.0, 0.075),
	vec3(0.925, 1.0, 0.075),
	vec3(0.075, 0.0, 0.925),
	vec3(0.075, 1.0, 0.925),

	// Diagonal 2 (1)
	vec3(0.075, 0.0, 0.925),
	vec3(0.075, 1.0, 0.925),
	vec3(0.925, 0.0, 0.075),
	vec3(0.925, 1.0, 0.075)
);

const vec2 faceUvs[4] = vec2[4](
	vec2(1, 0),
	vec2(1, 1),
	vec2(0, 0),
	vec2(0, 1)
);

const float face_lighting[10] = float[10](
	0.8,
	0.9,
	0.6,
	0.8,
	0.4,
	0.6,
	0.7,
	0.7,
	0.7,
	0.7
);

void main() {
	mat4 model;
	model[0][0] = 1.;
	model[1][1] = 1.;
	model[2][2] = 1.;
	model[3] = vec4(gs_in[0].offset, 1.);

	mat4 mvp = projection * view * model;
	mat4 mv = view * model;

	{
		voxelTexture = gs_in[0].voxelTexture;
		
		normal_out = faceNormals[gs_in[0].face] / 2. + 0.5;
		lighting_out = face_lighting[gs_in[0].face];

		uv_out = faceUvs[0];
		vec4 vert = (vec4(facePositions[gs_in[0].face * 4 + 0] * scale + vec3(1 - scale) * 0.5, 0.) + gl_in[0].gl_Position);
		gl_Position = mvp * vert;
		EmitVertex();

		uv_out = faceUvs[1];
		vert = (vec4(facePositions[gs_in[0].face * 4 + 1] * scale + vec3(1 - scale) * 0.5, 0.) + gl_in[0].gl_Position);
		gl_Position = mvp * vert;
		EmitVertex();

		uv_out = faceUvs[2];
		vert = (vec4(facePositions[gs_in[0].face * 4 + 2] * scale + vec3(1 - scale) * 0.5, 0.) + gl_in[0].gl_Position);
		gl_Position = mvp * vert;
		EmitVertex();

		uv_out = faceUvs[3];
		vert = (vec4(facePositions[gs_in[0].face * 4 + 3] * scale + vec3(1 - scale) * 0.5, 0.) + gl_in[0].gl_Position);
		gl_Position = mvp * vert;
		EmitVertex();

		EndPrimitive();
	}
}